/*
 * FSpot.Core ?
 *
 * Author(s):
 * 	Vasiliy Kirilichev <vasyok@gmail.com>
 *
 * This is free software. See COPYING for details.
 *
 */

using System;
using System.IO;
using System.Collections;

namespace FSpot {
public class ColorManagement {
		public static FSpot.PhotoImageView view;
		
		private static Cms.Profile display_profile;
		private static Cms.Profile destination_profile;
		private static Cms.Profile[] display_profile_list;

		private static string [] search_dir = { "/usr/share/color/icc", "~/.color/icc", "/usr/local/share/color/icc " };
		public static ArrayList ProfilesPaths = new ArrayList();
		
		public static Cms.Profile DisplayProfile {
			set {
				display_profile = value;
				//FIXME
				switch (display_profile.ProductName) {
				case "sRGB built-in - (lcms internal)":
					Preferences.Set (Preferences.DISPLAY_COLOR_PROFILE_PATH, "StandardRgb");
					break;
				case "rgb built-in - (lcms internal)":
					Preferences.Set (Preferences.DISPLAY_COLOR_PROFILE_PATH, "AlternateRgb");
					break;
				default:
					foreach (string ColorProfile in ProfilesPaths)
						if (new Cms.Profile(ColorProfile).ProductName == display_profile.ProductName)
							Preferences.Set (Preferences.DISPLAY_COLOR_PROFILE_PATH, ColorProfile);
					break;
				}
			}
			get { return display_profile; }
		}

		public static Cms.Profile DestinationProfile {
			set {
				destination_profile = value;
				//FIXME
				switch (destination_profile.ProductName) {
				case "sRGB built-in - (lcms internal)":
					Preferences.Set (Preferences.OUTPUT_COLOR_PROFILE_PATH, "StandardRgb");
					break;
				case "rgb built-in - (lcms internal)":
					Preferences.Set (Preferences.OUTPUT_COLOR_PROFILE_PATH, "AlternateRgb");
					break;
				default:
					foreach (string ColorProfile in ProfilesPaths)
					if (new Cms.Profile(ColorProfile).ProductName == destination_profile.ProductName)
						Preferences.Set (Preferences.OUTPUT_COLOR_PROFILE_PATH, ColorProfile);
					break;
					}
			}
			get { return destination_profile; }
		}		
		
		public static Cms.Profile[] StandartDisplayProfileList {
			set { display_profile_list = value; }
			get { return display_profile_list; }
		}
		
		public static void GetProfilesPaths (string path)
		{
			if (Directory.Exists (path)) {
				string[] ColorProfilList = System.IO.Directory.GetFiles (path, "*.icc");
					foreach (string ColorProfile in ColorProfilList)
						ProfilesPaths.Add(ColorProfile);
				string[] ColorProfilList2 = System.IO.Directory.GetFiles (path, "*.icm");
					foreach (string ColorProfile in ColorProfilList2)
						ProfilesPaths.Add(ColorProfile);
				string[] DirList = System.IO.Directory.GetDirectories (path);
					foreach (string dir in DirList)
						GetProfilesPaths (dir);
			}
		}
		
		private static void GetSettings ()
		{
			//FIXME: Display 
			Gdk.Screen screen = Gdk.Screen.Default;
			display_profile = Cms.Profile.GetScreenProfile (screen);
			if (display_profile == null) {
				try {
					display_profile = new Cms.Profile (Preferences.Get<string> (Preferences.DISPLAY_COLOR_PROFILE_PATH));
					//
					Gdk.Screen current_screen = Gdk.Screen.Default;
					FSpot.Widgets.CompositeUtils.SetScreenProfile(current_screen, display_profile);
				}
				catch {
					display_profile = Cms.Profile.CreateStandardRgb ();
					//Preferences.Set<string> (Preferences.DISPLAY_COLOR_PROFILE_PATH, ""); //fixme
				}
			}
			//Output
			switch (Preferences.Get<string> (Preferences.OUTPUT_COLOR_PROFILE_PATH)) {
			case "StandardRgb":
				destination_profile = Cms.Profile.CreateStandardRgb ();
				break;
			case "AlternateRgb":
				destination_profile = Cms.Profile.CreateAlternateRgb ();
				break;
			default:
				try {
					destination_profile = new Cms.Profile (Preferences.Get<string> (Preferences.OUTPUT_COLOR_PROFILE_PATH));
				}
				catch {
					destination_profile = Cms.Profile.CreateStandardRgb ();
				}
				break;
			}
		}
		
		public static void LoadSettings()
		{
			ProfilesPaths.Clear();
			foreach (string dir in search_dir)
				GetProfilesPaths (dir);
			ProfilesPaths.Sort();
			GetSettings();
			LoadStandartDisplayProfileList(); //?
		}
		
		public static void ReloadSettings()
		{
			GetSettings();
			LoadStandartDisplayProfileList(); //?
		}
		
		public static void LoadStandartDisplayProfileList()
		{
			FSpot.ColorManagement.StandartDisplayProfileList = new Cms.Profile [] {Cms.Profile.CreateStandardRgb (), display_profile};
		}
		
		public static Cms.Transform CreateTransform (Gdk.Pixbuf pixbuf, Cms.Profile image_profile)
		{
//			if (image_profile == null)
//				image_profile = Cms.Profile.CreateStandardRgb ();
//			
			Cms.Profile [] list = new Cms.Profile [] { image_profile, display_profile };
			
			Cms.Transform transform = new Cms.Transform (list,
			                                             PixbufUtils.PixbufCmsFormat (pixbuf),
			                                             PixbufUtils.PixbufCmsFormat (pixbuf),
			                                             Cms.Intent.Perceptual, 0x0000);
			return transform;
		}
		
		public static Cms.Transform CreateTransform ()
		{
			Cms.Transform transform = new Cms.Transform (StandartDisplayProfileList,
			                                             Cms.Format.Rgb8,
			                                             Cms.Format.Rgb8,
			                                             Cms.Intent.Perceptual, 0x0000);
			return transform;
		}
		
		public static void ApplyScreenProfile (Gdk.Pixbuf pixbuf)
		{
			Cms.Transform transform = new Cms.Transform (StandartDisplayProfileList,
			                                             PixbufUtils.PixbufCmsFormat (pixbuf),
			                                             PixbufUtils.PixbufCmsFormat (pixbuf),
			                                             Cms.Intent.Perceptual, 0x0000);
			PixbufUtils.ColorAdjust (pixbuf, pixbuf, transform);
		}
		
		public static void ApplyScreenProfile (Gdk.Pixbuf pixbuf, Cms.Profile image_profile)
		{
			if (image_profile == null)
				image_profile = Cms.Profile.CreateStandardRgb ();
			
			Cms.Profile [] list = new Cms.Profile [] { image_profile, display_profile };
			
			Cms.Transform transform = new Cms.Transform (list,
			                                             PixbufUtils.PixbufCmsFormat (pixbuf),
			                                             PixbufUtils.PixbufCmsFormat (pixbuf),
			                                             Cms.Intent.Perceptual, 0x0000);
			
			PixbufUtils.ColorAdjust (pixbuf, pixbuf, transform);
		}
		
//		public static void ApplyScreenProfile(Gdk.Pixbuf pixbuf, Gdk.Rectangle area)
//		{
//			Cms.Transform transform = new Cms.Transform (StandartDisplayProfileList,
//			                                             PixbufUtils.PixbufCmsFormat (pixbuf),
//			                                             PixbufUtils.PixbufCmsFormat (pixbuf),
//			                                             Cms.Intent.Perceptual, 0x0000);
//			PixbufUtils.ColorAdjust (pixbuf, pixbuf, area, transform);
//		}
		
//		public static void ApplyScreenProfile(Gdk.Pixbuf src, Gdk.Pixbuf dest)
//		{
//			Cms.Transform transform = new Cms.Transform (DisplayProfileList,
//			                                             PixbufUtils.PixbufCmsFormat (src),
//			                                             PixbufUtils.PixbufCmsFormat (src),
//			                                             Cms.Intent.Perceptual, 0x0000);
//			PixbufUtils.ColorAdjust (src, dest, transform);
//		}
		
		public static void ApplyPrinterProfile(Gdk.Pixbuf pixbuf, Cms.Profile image_profile)
		{
			if (image_profile == null)
				image_profile = Cms.Profile.CreateStandardRgb ();
			
			Cms.Profile [] list = new Cms.Profile [] { image_profile, destination_profile };
			
			Cms.Transform transform = new Cms.Transform (list,
			                                             PixbufUtils.PixbufCmsFormat (pixbuf),
			                                             PixbufUtils.PixbufCmsFormat (pixbuf),
			                                             Cms.Intent.Perceptual, 0x0000);
			PixbufUtils.ColorAdjust (pixbuf, pixbuf, transform);
		}
	}
}
